
import(FNN,mvtnorm)

  import("graphics")
  import("stats")
  import("utils")
  import("grDevices")
  import('MASS')
  import('glmnet')
  import('rje')
  import('text2vec')
  importFrom("data.table",'setDT','setkey')
  importFrom('car','vif')
  importFrom("sandwich","vcovHC")
  importFrom('gtools','defmacro')

S3method(coef,lmac)
S3method(vcov,lmac)
S3method(plot,rlm)
S3method(predict,knn)
S3method(predict,kNN)
S3method(predict,kNNallK)
S3method(plot,kmin)
S3method(predict,ovaknn)
S3method(plot,tuner)
S3method(predict,krsFit)
S3method(predict,penroseLM)
S3method(predict,penrosePoly)
S3method(predict,ridgePoly)

export(knnest,preprocessx,meany,mediany,vary,loclin, loclogit, predict.knn,nlshc,
   kmin,lmac,coef.lmac,vcov.lmac,pcac,loglinac,tbltofakedf,
   makeNA,ZerosToNAs,NAsTo0s,
   ovalogtrn,avalogtrn,avalogpred,
   doPCA,ridgePoly,replicMeans,PCAwithFactors,
   knntrn,ovaknntrn,predict.ovaknn,classadjust,
   knnFineTune,kNN,findOverallLoss,MAPE,probIncorrectClass,kNNxv,propMisclass,
   plotExpVars,exploreExpVars,multCols,kNNallK, 
   bestKperPoint, parvsnonparplot,nonparvsxplot,nonparvarplot,
   ridgelm,plot.rlm, boundaryplot,
   l2,l1, 
   mm,unscale,mmscale,constCols,catDFRow,
   confusion,factorTo012etc,allNumeric,
   penroseLM,predict.penroseLM,penrosePoly,predict.penrosePoly,ridgePoly, 
   ulist,getNamedArgs,discretize,
   toAllNumeric,stopBrowser,
   factorsToDummies,factorToDummies,dummiesToFactor,hasFactors,dummiesToInt,
   xyDataframeToMatrix,dummiesToInt,charsToFactors,hasCharacters,
   stdErrPred,getDFclasses,
   toSuperFactor,toSubFactor,
   prToFile,pythonBlankSplit,xyzPlot,
   fineTuning,fineTuningPar,partTrnTst,
   krsFit,krsFitImg,diagNeural,
   TStoX,TStoXmv,textToXY,textToXYpred)
